﻿namespace Bulldozer.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="tractor">Добавляемая установка</param>
        /// <returns></returns>
        public int Insert(T tractor)
        {
            return Insert(tractor, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="tractor">Добавляемая установкаь</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T tractor, int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= _maxCount)
            {
                // Позиция недопустима
                return -1;
            }
            if (Count >= _maxCount)
                return -1;
            _places.Insert(position, tractor);
            return position;
            
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            // TODO проверка позиции
            // Проверка позиции
            if ((position < 0) || (position > _maxCount))
            {
                // Позиция недопустима
                return false;
            }
            // TODO удаление объекта из массива, присвоив элементу массива значение null
            _places.RemoveAt(position);
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            // TODO проверка позиции
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                _places[position] = value;
            }
        }
        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetBulldozer(int? maxTractor = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxTractor.HasValue && i == maxTractor.Value)
                {
                    yield break;
                }
            }
        }
    }
}

